% TODO 
% - find some way to record fact that EEG and MEG channels have different scales
% - replace labels for known EEG locations
% - maybe change meg labels to grad/megplaner/mag or something similar
% - get rid of bipolar channels? or do in bit which stitches runs together?
% - convert to average reference?

function [EEG] = readElekta(inFilePath)
%inFilePath = '/mnt/data/brianEEG/megPilot/murphy_brian/091105/en6662orth_part1.fif'
% This function imports data from the Elekta format 
% correct electrode position data, filters data at 1 and 50 HZ, runs ICA (having reduced dimensions to fit length of data).
% The ICA versions are saved as EEGLAB set files.
% did it this way, cos downsampled data had few points, meaning fewer ICA components achievable. Now trying to see if get better decomposition with high sampling rate data.

tic
datestr(now) % print start time

% PATHS, INPUT
addpath('/home/software/matlabToolboxes/eeglab/eeglab7_1_3_14b/')
addpath('/home/software/matlabToolboxes/mne/matlab/toolbox/')
x = findstr('/',inFilePath);
inFileName = inFilePath(x(end)+1:end)
% inDir = '/mnt/data/brianEEG/rawData/';
% outDir = '/mnt/data/brianEEG/eeglabData/';
% inFilePath = strcat(inDir, inFileName);

% FILTERING
filter = [1 50];
downSample = (filter(2))*(2.5);

% OUTPUT
icaSetName = sprintf('%s chanInfo filt%i-%i ICAed resamp%i', inFileName, filter(1), filter(2), downSample)
icaFilePath = sprintf('%s_chanInfo_filt%i-%i_ICAed_resamp%i.set', strrep(inFilePath, '.fif', ''), filter(1), filter(2), downSample)

if exist(icaFilePath) % CHECK IF THIS FILE ALREADY PROCESSED
	fprintf('### skipping %s, cos seems already taken care of\n', inFileName);
	return;
end

% LOAD RAW DATA
% preload eeglab
eeglab redraw
close(findobj('tag', 'EEGLAB')) % close menu window

fprintf('### will load %s\n', inFileName);
EEG = pop_fileio(inFilePath);   

% FIX SCALES TO MAKE VIEWABLE
fprintf('\t### rescaling EEG, gradiometer and magnetometer signals to have similar amplitudes\n');
fixElektaScales

% ADD CHANNEL LOCATION DATA
fprintf('\t### fixing electrode labels, locations\n');
fixElektaChannels

% REMOVE TRIGGER CHANNELS
fprintf('\t### removing trigger channels, and fixing events\n');
% following doesn't work, though should according to docs
%EEG = pop_select( EEG, 'nochannel',eeg_chantype(EEG,'trigger'));
% ... work around ...
tIndices = eeg_chantype(EEG,'trigger');
tLabels = {};
for t = tIndices
    tLabels = [tLabels EEG.chanlocs(t).labels];
end
EEG = pop_select( EEG, 'nochannel',tLabels);
% strip out response from triggers, and make uninteresting triggers
% invisible
fixElektaEvents

% FIX ELEKTA EVENTS
%

% FILTER HIGH AND LOW PASS
fprintf('\t### applying high-pass and low pass FIR filters at %i and %i Hz\n', filter(1), filter(2));
for c=1:EEG.nbchan
    c
    EEG.data(c,:) = eegfilt(EEG.data(c,:),EEG.srate,filter(1), 0);
    EEG.data(c,:) = eegfilt(EEG.data(c,:),EEG.srate,0, filter(2));
end
% EEG = pop_eegfilt( EEG, filter(1), 0, [], [0]); % these used too much
% memory
% EEG = pop_eegfilt( EEG, 0, filter(2), [], [0]);

% ??? % do ICA on high-sampling rate data, and then copy these components
% accross?
% ??? try 'fastica' to find first 30 or so components, without (I assume)
% PCA first?
% ??? should eye artefacts come out as easily for EEG/MEG? not if there's a
% lot of movement?
% ??? run separately for eeg and meg? to fix movement problem somewhat?

% JUST IN CASE ...
fprintf('\t### saving temp file in case ICA fails: %s\n', [icaFilePath '.tmp']);
EEG = pop_saveset( EEG,  [icaFilePath '.tmp']);


% RUN ICA
fprintf('\t### doing ICA\n');
% no longer doing pre-ica pca - this resulted in really noisy looking
% components, whereas dropping this step worked better with OrthoB5 (albeit
% on resampled data). Now trying on full data. If unsuccessful, next steps
% to try are fastica of first 30-ish components (high res data), and full
% ica on low res data.
% Once had problem with ICA non converging, but haven't replicated it...
% note that extended is recommended for noisy data
EEG = pop_runica(EEG,  'icatype', 'binica', 'options',{ 'extended',1});

% DOWNSAMPLE
fprintf('\t### downsampling to %i Hz\n', downSample);
EEG = pop_resample( EEG, downSample);

% FILTERED, ICAed DATA
fprintf('\t### saving "%s" as "%s"\n', icaSetName, icaFilePath);
EEG = pop_editset(EEG, 'setname',  icaSetName);
EEG = eeg_checkset(EEG);
EEG = pop_saveset( EEG,  icaFilePath);

% PLOT ICA COMPS AND SAVE IMAGE
% plotNumComps = size(EEG.icaweights,1);
% if plotNumComps > 36
%     plotNumComps = 36;
% end
% pop_topoplot(EEG,0, [1:plotNumComps] , [icaSetName ' (' num2str(plotNumComps) ' of first ' num2str(size(EEG.icaweights,1)) ' comps)'],[6 6] ,0, 'electrodes', 'off');
% saveas(gcf, [icaFilePath '.png'])
% close gcf

datestr(now)
toc

%eeglab redraw


exit;
